/* 
 * TclTest.java --
 *
 *	This file is used to test the tcljava package.
 *
 * Copyright (c) 1995 by Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

import sun.tools.zip.*;
import java.util.Enumeration;
import java.util.Date;
import tcl.*;

/*
 * This first callback object is used to invoke the widget demo, running
 * under Java.
 */
class Callback1 extends Command {
    /*
     * All callback objects have an invoke method for when they're called,
     * a deleted method for when they are removed from the interpreter,
     * and a finalized destructor method.
     */
    public int invoke(Interp interp, Value args[]) {
	/*
	 * We get a new result object each time this command is invoked
	 * because we're going to do an eval.  If we weren't we would
	 * simply return the interp.TCL_OK or interp.TCL_ERROR constant 
	 * as appropriate.
	 */
	Result result = new Result();
	/*
	 * Eval the command, and put the result in the result object.
	 */
	interp.eval("source /proj/tcl/install/lib/tk4.1/demos/widget",
		result);
	/*
	 * Make sure the window can be seen.
	 */
	if (result.returnCode == Interp.TCL_OK) {
	    interp.eval("wm deiconify .", result);
	}
	
	/*
	 * Return the numeric result.
	 */
	return result.returnCode;
    }
    
    /* 
     * This method is called when the command is deleted from the
     * interpreter.
     */
    public void deleted(Interp interp) {
	System.out.println("deleting callback1...");
    }
    protected void finalize() { System.out.println("finalized Callback1"); }
}

class Callback2 extends Command {
    /*
     * Note that for this callback, we don't return a result, and
     * therefore don't need a Result object.
     */
    public int invoke(Interp interp, Value args[]) {
	Runtime rt = Runtime.getRuntime();
	rt.gc();
	return Interp.TCL_OK;
    }
    public void deleted(Interp interp) {
	System.out.println("deleting callback2...");
    }
    protected void finalize() { System.out.println("finalized Callback2"); }
}

class zipCallback extends Command {
    /*
     * A slightly more intricate command.  This command returns a tcl list of
     * lists, that contain information about the elements of the particular
     * zip file.
     * 
     * The format of the four elements of the each sub-list are:
     * { { name } { '/' delineated path, if given } \
     *   { file size in bytes } { Date modified } }
     */
     public int invoke(Interp interp, Value args[]) {
	ZipFile zf;
	Object tmpobj;
	String listing = "";
        if (args.length != 2) {
	    /*
	     * For this example, errors are handled here, in the invocation,
	     * rather than later, as in the other two callbacks.
	     */
	    interp.setResult(new Value(" wrong # args: should be \"zipfilelist filename\""));
	    return interp.TCL_ERROR;
	}
	try {
	    /*
	     * ZipFile can return either of the two exceptions below,
	     * so we catch both of them.
	     */
	    zf = new ZipFile(args[1].toString());
	} catch (sun.tools.zip.ZipFormatException e) {
	    interp.setResult(new Value("Zip file has bad format."));
	    return interp.TCL_ERROR;
	} catch (java.io.IOException e) {
	    interp.setResult(new Value("IO Error (maybe the file doesn't exist?)"));
	    return interp.TCL_ERROR;
	}
	/*
	 * Good! We successfully opened the zip file, and read its table
	 * of contents.  Let's now turn that into a tcl list.  Note that
	 * for our purposes, this is a list of lists, each of which have
	 * the four elements mentioned above.
	 */
	for (Enumeration e = zf.entries();e.hasMoreElements() ;) {
	    tmpobj = e.nextElement();
	    Date d = new Date((((ZipEntry)(tmpobj)).lastModified()));
	    listing += "{ {"+((ZipEntry)(tmpobj)).getPath()+"} "+
	               " {"+((ZipEntry)(tmpobj)).getName()+"} "+
		       " {"+Long.toString((((ZipEntry)(tmpobj)).length()),10) +"} "+
		       " {"+d.toString()+"} } ";
	}
	/* 
	 * Set the Result to be what we've worked so hard for:
	 */
	interp.setResult(new Value(listing));
	return interp.TCL_OK;
    }
    public void deleted(Interp interp) {
	System.out.println("deleting zipCallback...");
    }
}

class TclTest {

    static {
       System.loadLibrary("jtk");
    }

    /*
     * Eval the given command, printing it out on stdout, and 
     * also showing whether it was successful, or if it failed.
     */
    public static void evalPrint(Interp interp, String s) {
	Result r = new Result();
	if (interp.eval(s,r) != Interp.TCL_OK) {
	    System.out.println("eval(" + s + ") failed: " + r.value);
	} else {
	    System.out.println("eval(" + s + ") succeed: " + r.value);
	}
    }

    public static void main(String args[]) {
	/* 
	 * We create an interpreter, and initialize it to handle
	 * tk.
	 */
	Interp interp = new Interp();
	TkApplication.tkInit(interp);

	/* 
	 * Create the 3 commands in TCL, and associate them with the
	 * callback objects.
	 */
	interp.createCommand("callback1", new Callback1());
	interp.createCommand("callback2", new Callback2());
	interp.createCommand("zipfilelist", new zipCallback());
	
	/*
	 * Set up our demo application.
	 */
	evalPrint(interp, "toplevel .java");
	evalPrint(interp, "wm withdraw .");
	evalPrint(interp, "wm title .java {Java Demonstration}");
	evalPrint(interp, "button .java.b -text demo -command callback1");
	evalPrint(interp, "button .java.b2 -text delete -command {rename callback1 {}}");
	evalPrint(interp, "button .java.b3 -text gc -command callback2");
	evalPrint(interp, "button .java.b4 -text {Zip Browser Demo} -command {source zipbrowser.tcl}");
	evalPrint(interp, "pack .java.b .java.b2 .java.b3 .java.b4");

	/*
	 * Handle all the events that happen.  We put the event loop here, 
	 * instead of calling Tk_MainLoop so that we can allow threads.
	 * 
	 * BEWARE! Threads are dangerous with TK.  You *must* ensure that
	 * only *one* thread is doing tk/tcl calls, and is being called
	 * from tk/tcl.  If you have more than one thread doing anything
	 * with tk/tcl, you'll probably notice with the heinous core dumps
	 * your brand new app starts dumping.
	 * 
	 * You've been warned.
	 */
	TkApplication.mainLoop();
	
	/*
	 * We *only* get to this point when all the main windows in the
	 * Tk Application goes away, as contrasted with AWT that exits
	 * the main thread as soon as things are initialized.
	 */
	System.out.println("exiting...");
    }
}
