/* 
 * tclJava.c --
 *
 *	This file implements the native methods for the tcl.Interp class.
 *
 * Copyright (c) 1995 by Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) tclJava.c 1.2 95/12/19 14:34:48
 */
#include <tcl.h>

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <oobj.h>
#include <interpreter.h>
#include <common_exceptions.h>
#include <java_lang_String.h>
#include <javaString.h>
#include <typecodes.h>

#include "tcl_classes.h"

typedef struct CommandInfo {
    Htcl_Command *cmd;
    Htcl_Interp *interp;
} CommandInfo;

static ClassClass *value_class;

/*
 * This is the callback that all TCL commands written in Java actually call.
 * Since all callback objects have the invoke method, this is not a problem.
 */
static int
tcl_invokeCmd(ClientData clientData, Tcl_Interp *interp,
	 int argc, char *argv[])
{
    CommandInfo *info = (CommandInfo *) clientData;
    HArrayOfObject *java_args;
    struct execenv *ee = EE();
    int i, n, ret;

    /*
     * Collect all the tcl arguments, and put them into a new Java
     * array.
     * 
     * God bless garbage collection.
     */
    java_args = (HArrayOfObject *) ArrayAlloc(T_CLASS, argc);
    for (i = 0; i < argc; i++) {
	unhand(java_args)->body[i] = (HObject *)execute_java_constructor(ee,
		0, value_class, "(Ljava/lang/String;)",
		makeJavaString(argv[i], strlen(argv[i])));
    }

    /*
     * invoke the java command.
     */
    ret = (int) execute_java_dynamic_method(ee, (HObject *)info->cmd, "invoke",
	    "(Ltcl/Interp;[Ltcl/Value;)I", info->interp, java_args);
    if (exceptionOccurred(ee)) {
	interp->result = "Exception";
	return TCL_ERROR;
    }
    return ret;
}

/*
 * Oops! A TCL command was deleted.  Let the java object know.
 */
static void
tcl_deleteCmd(ClientData clientData)
{
    CommandInfo *info = (CommandInfo *) clientData;
    execute_java_dynamic_method(0, (HObject *)info->cmd, "deleted",
	    "(Ltcl/Interp;)V", info->interp);
    execute_java_dynamic_method(0, (HObject *)unhand(info->interp)->commands,
	    "removeElement", "(Ljava/lang/Object;)Z", info->cmd);
    ckfree(info);
}    

/*
 * Evaluate a String, returning the result code as necessary.
 */
long
tcl_Interp_eval(Htcl_Interp *interpH, HString *argH, Htcl_Result *resultH)
{
    Classtcl_Result *result = unhand(resultH);
    Tcl_Interp *interp = (Tcl_Interp *) unhand(interpH)->interp;

    result->returnCode = Tcl_Eval(interp, makeCString(argH));
    unhand(result->value)->string = makeJavaString(interp->result,
	    strlen(interp->result));
    return result->returnCode;
}

/*
 * Create the command in TCL-space, making the function defined
 * above "tcl_deleteCmd" the callback command.
 */
void
tcl_Interp_createCommand(Htcl_Interp *interpH, HString *nameH,
	Htcl_Command *cmdH)
{
    Tcl_Interp *interp = (Tcl_Interp *) unhand(interpH)->interp;
    CommandInfo *info = (CommandInfo *) ckalloc(sizeof(CommandInfo));
    
    info->cmd = cmdH;
    info->interp = interpH;

    execute_java_dynamic_method(0, (HObject *)unhand(interpH)->commands,
	    "addElement", "(Ljava/lang/Object;)V", cmdH);

    Tcl_CreateCommand(interp, makeCString(nameH), tcl_invokeCmd,
	    (ClientData) info, tcl_deleteCmd);
}

/*
 * Sets the TCL interp->result string.
 */
void
tcl_Interp_setResult(Htcl_Interp *interpH, Htcl_Value *valueH)
{
    Tcl_Interp *interp = (Tcl_Interp *) unhand(interpH)->interp;

    Tcl_ResetResult(interp);
    Tcl_AppendResult(interp, makeCString(unhand(valueH)->string));
}

/*
 * Initialize TK.
 */
void
tcl_Interp_create(Htcl_Interp *tclH)
{
    Classtcl_Interp *tcl = unhand(tclH);

    /*
     * Initialize static variables, if necessary.
     */

    if (value_class == NULL) {
	value_class = FindClass(0, "tcl/Value", TRUE);
    }

    tcl->interp = (int) Tcl_CreateInterp()
;
    if (tcl->interp == 0) {
	SignalError(0, JAVAPKG "UnsatisfiedLinkException", "TCL");
	return;
    }

    Tcl_Init((Tcl_Interp *)(tcl->interp));

    return;
}

/*
 * Do one event, passing appropriate flags as necessary.
 */
long
tcl_Interp_doOneEvent(struct Htcl_Interp *interpH, long flags)
{
#ifdef CREATEMAINWINDOWHACK
    return Tk_DoOneEvent(flags);
#else
    return Tcl_DoOneEvent(flags);
#endif
}

